__author__ = 'vinay'
from django.conf import settings

from .models import Content
from .views import cms_page_index


class Middleware(object):

    def process_response(self, request, response):
        # ignore all ajax, static and media requests
        if request.is_ajax() or \
                request.path.startswith(settings.STATIC_URL or '///') or \
                request.path.startswith(settings.MEDIA_URL or '///'):
            return response

        if response.status_code != 404:
            return response

        try:
            cms_page = Content.objects.get(url=request.path)
        except Content.DoesNotExist:
            return response
        else:
            return cms_page_index(request, cms_page=cms_page)



__author__ = 'vinay'
from django.core.exceptions import ValidationError
from django.http.response import Http404, HttpResponse
from django.template import TemplateSyntaxError

from .models import Content
from .utils import can_edit_content, parse_cms_template
from .validators import validate_and_get_template


def render(template_name,
           template_context=None, content_type=None, status=None,
           using=None, request=None, cms_context=None):
    """
    Validates the given template and parses it's content
    returning HttpResponse or raising Http404 for syntax and template errors
    that are included in the cms_context

    This function is a hybrid of django's render and render_to_response by removing the deprecated arguments
    This also takes an optional parameter `cms_context` which is used to render the template.
    """
    template = validate_and_get_template(template_name, using=using)
    content = template.render(context=template_context, request=request)

    if request is not None:
        try:
            cms_page = Content.objects.get(url=request.path)
        except Content.DoesNotExist:
            pass
        else:
            if can_edit_content(request.user):
                cms_context = cms_page.auth_data
            elif cms_page.is_live:
                return HttpResponse(cms_page.html, content_type, status)

    if cms_context is not None:
        if isinstance(cms_context, dict):
            try:
                content = parse_cms_template(content, cms_context,
                                             publish=False)
            except (ValidationError, TemplateSyntaxError):
                raise Http404()
        else:
            raise ValueError('cms_context should be an instance of dict')

    return HttpResponse(content, content_type, status)



__author__ = 'vinay'
import json
import traceback

from django.http.response import HttpResponseForbidden, HttpResponse, Http404, JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.http import require_POST

from .models import Content
from .response import render
from .utils import NAMESPACE_DELIMITER, user_passes_test, can_edit_content


def cms_page_index(request, cms_page):
    if can_edit_content(request.user):
        return render(
            template_name=cms_page.template,
            cms_context=cms_page.auth_data,
            request=request,
            template_context={'view': 'author',
                              'cms_data': json.dumps(cms_page.auth_data),
                              'id': cms_page.id,
                              'namespace_delimiter': json.dumps(NAMESPACE_DELIMITER)})

    if cms_page.is_public:
        return HttpResponse(cms_page.html)

    raise Http404("page not found")


@require_POST
@csrf_protect
@user_passes_test()
def save(request, save_type):
    if not request.is_ajax():
        return HttpResponseForbidden()

    post_data = request.POST
    if any(_ not in post_data for _ in ('auth_data', 'cms_page_id')) or save_type not in ('draft', 'publish'):
        return JsonResponse({'success': False}, status=400)

    cms_page = get_object_or_404(Content, id=post_data['cms_page_id'])

    try:
        content = json.loads(post_data['auth_data'])
    except:
        return JsonResponse({'success': False, 'message': 'Invalid JSON object'}, status=400)
    else:
        cms_page.auth_data.update(content)
        try:
            render(cms_page.template, cms_context=cms_page.auth_data)
        except Exception as e:
            return JsonResponse({'success': False,
                                 'message': 'Unable to parse the new content.\n'
                                 'Please resolve the issues and try again',
                                 'exception': e,
                                 'traceback': traceback.format_exc()})
        else:
            if save_type == 'draft':
                cms_page.save()
            else:
                cms_page.publish_cms_content()
            return JsonResponse({'success': True})



import datetime

from django.db import models


class ContentManager(models.Manager):

    def published(self):
        return self.get_queryset().filter(status__gte=2, publish__lte=datetime.datetime.now())

    def static_pages(self):
        return self.get_queryset().filter(category__id=1)



__author__ = 'vinay'
import datetime
from functools import wraps
import os
import re

from bs4 import BeautifulSoup, Tag
from mistune import markdown

from django.contrib.auth.models import AnonymousUser
from django.core.cache import cache
from django.core.exceptions import PermissionDenied
from django.http.request import HttpRequest
from django.template import TemplateSyntaxError
from django.utils.decorators import available_attrs
from django.utils.text import slugify

from .settings import PAGE_CACHE_TIMEOUT
from .validators import validate_and_get_template, ValidationError

dum_request = HttpRequest()
dum_request.user = AnonymousUser()

CMS_ATTRIBUTES = ['data-cms-attr', 'data-cms-content', 'data-cms-include']

attr_re = re.compile(r'^(?:(?:[a-z-]+:[a-z_0-9]+)+(?:\|(?:[a-z-]+:[a-z_0-9]+))*)*$', re.IGNORECASE)
content_re = re.compile(r'^(?:md:)?([a-z_0-9])+$', re.IGNORECASE)
include_html_re = re.compile(r'^(?:(?:[a-z_0-9])+:)?.+$', re.IGNORECASE)
namespace_re = re.compile(r'^(?:[a-z_0-9])+$', re.IGNORECASE)

NAMESPACE_DELIMITER = '-'

# only 'html.parser' doesn't bother adding <html> tag for html components
HTML_PARSER = 'html.parser'


def parse_cms_template(html, dictionary, parent_namespace='', publish=False):
    """
    Refer to tests for cms syntax

    :param html: Html to be parsed using cms syntax
    :type html: str
    :param dictionary: Dictionary that is to be used to parse the cms attributes in template
    :type dictionary: dict
    :param parent_namespace: Namespace of the html content to be parsed (if any)
    :type parent_namespace: str
    :param publish: This will hide sensitive info while rendering for public usage
    :type publish: bool
    :rtype : str
    """
    soup = BeautifulSoup(html, features=HTML_PARSER)

    for tag in soup.find_all(attrs={'data-cms-include': include_html_re}):
        namespace = get_namespace(tag, parent_namespace=parent_namespace)
        include_value = tag.attrs.pop('data-cms-include')
        if ':' in include_value:
            local_namespace, default_template_name = include_value.split(':', 1)
        else:
            try:
                local_namespace = tag.attrs['data-cms-namespace']
                default_template_name = include_value
            except:
                raise TemplateSyntaxError(
                    'value of data-cms-include should be of the form {namespace}:{template path}'
                    'if namespace is not specified then another attribute data-cms-namespace should be defined'
                )

        namespace += NAMESPACE_DELIMITER + local_namespace if namespace else local_namespace

        template_name = dictionary.get(namespace, default_template_name)

        if template_name.endswith('.html'):
            template_name = template_name[:-5]

        try:
            include_template = validate_and_get_template(template_name)
        except ValidationError:
            include_template = validate_and_get_template(
                default_template_name[:-5] if default_template_name.endswith('.html') else default_template_name
            )

        include_html = include_template.render(request=dum_request)

        tag.attrs['data-cms-namespace'] = local_namespace
        if not publish:
            tag.attrs['data-cms-include'] = template_name

        new_tag = Tag(soup, name=tag.name, attrs=tag.attrs)
        new_tag.insert(0, BeautifulSoup(include_html, features=HTML_PARSER))
        tag.replaceWith(new_tag)

    # soup does not recognize the changes made in above loop unless I do this
    # Also do not move it inside the loop. It will mess up the variable scoping
    soup = BeautifulSoup(soup.encode_contents(), features=HTML_PARSER)

    for tag in soup.find_all(attrs={'data-cms-attr': attr_re}):
        _ns = get_namespace(tag, parent_namespace=parent_namespace)
        attrs = tag['data-cms-attr'].split('|')

        for attr in attrs:
            attr_name, key = attr.split(':', 1)
            key = _ns + NAMESPACE_DELIMITER + key if _ns else key

            if key in dictionary:
                tag[attr_name] = dictionary[key]

    soup = BeautifulSoup(soup.encode_contents(), features=HTML_PARSER)

    for tag in soup.find_all(attrs={'data-cms-content': content_re}):
        _ns = get_namespace(tag, parent_namespace=parent_namespace)
        key = tag['data-cms-content']
        md = False

        if key.startswith('md:'):
            key = key[3:]
            md = True

        key = _ns + NAMESPACE_DELIMITER + key if _ns else key

        if key in dictionary:
            content = dictionary[key]
        else:
            content = tag.encode_contents()
            if not any(_ in content for _ in CMS_ATTRIBUTES):
                continue

        new_tag = Tag(soup, name=tag.name, attrs=tag.attrs)
        if any(_ in content for _ in CMS_ATTRIBUTES):
            content = parse_cms_template(content, dictionary, parent_namespace=key)

        if md:
            content = markdown(content, False)

        new_tag.insert(0, BeautifulSoup(content, features=HTML_PARSER))
        tag.replace_with(new_tag)

    soup = BeautifulSoup(soup.encode_contents(), features=HTML_PARSER)
    # don't use soup.prettify as it will insert empty spaces inside textarea
    return soup.encode_contents()


def get_namespace(tag, parent_namespace=''):
    ancestor_namespaces = [parent.attrs.get('data-cms-namespace') or parent.attrs.get('data-cms-content')
                           for parent in tag.find_parents(is_namespace_parent)]

    if parent_namespace:
        ancestor_namespaces.append(parent_namespace)

    return NAMESPACE_DELIMITER.join(reversed(ancestor_namespaces))


def is_namespace_parent(tag):
    attrs = tag.attrs
    if namespace_re.match(attrs.get('data-cms-namespace', '')) or content_re.match(attrs.get('data-cms-content', '')):
        return True

    return False


def get_public_data(cms_page, request=dum_request):
    """
    Generates the public data for the given cms page. Raises exceptions if unable to process the data.
    :type cms_page: BaseCmsPage
    :type request: HttpRequest
    :rtype : str
    """
    # Removed all the exception handling
    # as data is being validated at all levels before saving into the model
    cms_template = validate_and_get_template(cms_page.template)
    html = cms_template.render(request=request)
    return parse_cms_template(html, cms_page.public_data,
                              publish=True)


def can_edit_content(user):
    return user.has_perm('cfblog.change_content')


def user_passes_test(test_func=can_edit_content):
    """
    Similar to user_passes_test in django.contrib.auth.decorators.
    Instead of redirecting it just raises PermissionDenied.

    :param test_func: Should take a User object and return a bool
    :type test_func: function
    """
    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):
            if test_func(request.user):
                return view_func(request, *args, **kwargs)
            else:
                raise PermissionDenied
        return _wrapped_view
    return decorator


def cacheable(cache_key, timeout=PAGE_CACHE_TIMEOUT):
    """ Usage:

    class SomeClass(models.Model):
        # fields [id, name etc]

        @cacheable("SomeClass_get_some_result_{id}")
        def get_some_result(self):
            # do some heavy calculations
            return heavy_calculations()

        @cacheable("SomeClass_get_something_else_{name}")
        def get_something_else(self):
            return something_else_calculator(self)

    :type cache_key: unicode
    """
    def decorator(func):
        @wraps(func, assigned=available_attrs(func))
        def _wrapped_view(self):
            key = cache_key.format(**self.__dict__)
            if key in cache:
                return cache.get(key)
            res = func(self)
            cache.set(key, res, timeout)
            return res
        return _wrapped_view
    return decorator


def stale_cache(cache_key):
    """
    Removes the existing cache

    Usage:

    class SomeClass(models.Model):
        # fields
        name = CharField(...)

        @stale_cache("SomeClass_some_key_that_depends_on_name_%(name)")
        @stale_cache("SomeClass_some_other_key_that_depends_on_name_%(name)")
        def update_name(self, new_name):
            self.name = new_name
            self.save()
    """
    def decorator(func):
        @wraps(func, assigned=available_attrs(func))
        def _wrapped_view(self, *args, **kw):
            try:
                key = cache_key.format(**self.__dict__)
            except KeyError:
                # This happens when the model is being saved for the first time.
                pass
            else:
                cache.delete(key)
            finally:
                return func(self, *args, **kw)
        return _wrapped_view
    return decorator


def get_image_path(instance, filename):
    now = datetime.datetime.now()
    monthdir = now.strftime("%Y-%m")
    fname, ext = os.path.splitext(filename)
    new_fname = slugify(fname)
    newfilename = "%s-%s.%s" % (new_fname, now.strftime("%I%M%S"), ext)
    path_to_save = "uploads/blog/%s/%s" % (monthdir, newfilename)
    return path_to_save


def get_file_path(self, instance, filename):
    now = datetime.datetime.now()
    monthdir = now.strftime("%Y-%m")
    fname, ext = os.path.splitext(filename)
    new_fname = slugify(fname)
    newfilename = "%s-%s.%s" % (new_fname, now.strftime("%I%M%S"), ext)
    path_to_save = "uploads/pdfs/%s/%s" % (monthdir, newfilename)
    return path_to_save



__author__ = 'vinay'

from django.test import TestCase

from .utils import parse_cms_template, NAMESPACE_DELIMITER


class CMSTests(TestCase):
    def test_basic_features(self):
        html = """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">hello</h1>
                </body>
            </html>
        """
        out = parse_cms_template(html, {})
        self.assertHTMLEqual(html, out)

        out = parse_cms_template(html, {"h1": "foo"})
        self.assertHTMLEqual(
            out, """
                <html>
                    <head>
                    </head>
                    <body>
                        <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">foo</h1>
                    </body>
                </html>
            """
        )

        self.assertHTMLEqual(
            parse_cms_template(html, {"id": "new_id"}),
            """
                <html>
                    <head>
                    </head>
                    <body>
                        <h1 data-cms-content="h1" data-cms-attr="id:id" id="new_id">hello</h1>
                    </body>
                </html>
            """
        )

    def test_advanced_features(self):
        html = """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">hello</h1>
                </body>
            </html>
        """
        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-content="text"></div>'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-content="text"></div>
                    </h1>
                </body>
            </html>
        """)

        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-content="text"></div>',
             'h1' + NAMESPACE_DELIMITER + 'text': 'replaced'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-content="text">replaced</div>
                    </h1>
                </body>
            </html>
        """)

        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-attr="id:div_id" id="default">hi</div>',
             'h1' + NAMESPACE_DELIMITER + 'div_id': 'replaced'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-attr="id:div_id" id="replaced">hi</div>
                    </h1>
                </body>
            </html>
        """)

        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-content="content" data-cms-attr="id:div_id" id="default">hi</div>',
             'h1' + NAMESPACE_DELIMITER + 'div_id': 'replaced',
             'h1' + NAMESPACE_DELIMITER + 'content': 'new content'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-content="content" data-cms-attr="id:div_id" id="replaced">new content</div>
                    </h1>
                </body>
            </html>
        """)

    def test_snippet_insertions(self):
        html = """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">hello</h1>
                </body>
            </html>
        """
        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-include="namespace:cms_templates/snippet"></div>'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet" data-cms-namespace="namespace">
                    hehehe
                    <div data-cms-content="md:text">Hi</div>
                    </div>
                    </h1>
                </body>
            </html>
        """)

        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-include="namespace:cms_templates/snippet"></div>',
             NAMESPACE_DELIMITER.join(('h1', 'namespace', 'text')): '##Heading##'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet" data-cms-namespace="namespace">
                    hehehe
                    <div data-cms-content="md:text"><h2>Heading</h2></div>
                    </div>
                    </h1>
                </body>
            </html>
        """)

        out = parse_cms_template(
            html,
            {'h1': '<div data-cms-include="namespace:cms_templates/snippet_2"></div>',
             NAMESPACE_DELIMITER.join(('h1', 'namespace', 'text2')): '##Heading##'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    <div data-cms-include="cms_templates/snippet_2" data-cms-namespace="namespace">
                    hehehe2
                    <div data-cms-content="md:text2"><h2>Heading</h2></div>
                    </div>
                    </h1>
                </body>
            </html>
        """)

        out = parse_cms_template(
            html,
            {'h1': '##Heading##\n'
                   '<div data-cms-include="namespace:cms_templates/snippet_2"></div>',
             NAMESPACE_DELIMITER.join(('h1', 'namespace', 'text2')): '##Heading##'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="h1" data-cms-attr="id:id" id="orig">
                    ##Heading##
                    <div data-cms-include="cms_templates/snippet_2" data-cms-namespace="namespace">
                    hehehe2
                    <div data-cms-content="md:text2"><h2>Heading</h2></div>
                    </div>
                    </h1>
                </body>
            </html>
        """)

        html = """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="md:h1" data-cms-attr="id:id" id="orig">hello</h1>
                </body>
            </html>
        """
        out = parse_cms_template(
            html,
            {'h1': '##Heading##\n'
                   '<div data-cms-include="namespace:cms_templates/snippet_2"></div>',
             NAMESPACE_DELIMITER.join(('h1', 'namespace', 'text2')): '##Heading##'}
        )
        self.assertHTMLEqual(out, """
            <html>
                <head>
                </head>
                <body>
                    <h1 data-cms-content="md:h1" data-cms-attr="id:id" id="orig">
                    <h2>Heading</h2>
                    <div data-cms-include="cms_templates/snippet_2" data-cms-namespace="namespace">
                    hehehe2
                    <div data-cms-content="md:text2"><h2>Heading</h2></div>
                    </div>
                    </h1>
                </body>
            </html>
        """)



from __future__ import unicode_literals

__author__ = 'vinay'

import datetime

from django.conf import settings
from django.db import models
from django.db.models import Q
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from jsonfield import JSONField
from tagging.fields import TagField

from .managers import ContentManager
from .utils import cacheable, stale_cache, get_public_data
from .validators import validate_url_path, validate_and_get_template


class Category(models.Model):
    """Category model."""
    title = models.CharField(_('title'), max_length=100, unique=True, db_index=True)
    description = models.TextField(_('description'))

    class Meta:
        verbose_name_plural = _('categories')
        ordering = ('title',)

    def __unicode__(self):
        return self.title

    def delete(self, using=None):
        assert self.pk is not 1, (
            'Will not delete {} category. Shit breaks if it happens'.format(self.title)
        )
        super(Category, self).delete(using=using)


def cms_authors():
    User = get_user_model()
    users = User.objects.filter(
        Q(user_permissions__codename='change_content', user_permissions__content_type__app_label='cfblog') |
        Q(groups__permissions__codename='change_content', groups__permissions__content_type__app_label='cfblog') |
        Q(is_superuser=True)).distinct()

    return {'pk__in': users.values('pk')}


class Content(models.Model):
    STATUS_CHOICES = (
        (1, _('Draft')),
        (2, _('Public')),
    )
    url = models.CharField(_('url path'),
                           max_length=255, db_index=True, unique=True, validators=[validate_url_path])
    template = models.CharField(verbose_name=_('template'), max_length=255, validators=[validate_and_get_template])
    status = models.IntegerField(_('status'), choices=STATUS_CHOICES, default=1)
    publish = models.DateTimeField(_('publish'), default=datetime.datetime.now)
    title = models.CharField(_('title'), max_length=200)
    category = models.ForeignKey('Category')
    tags = TagField()
    tease = models.TextField(_('tease'), blank=True,
                             help_text=_('Concise text suggested. Does not appear in RSS feed.'))
    # TODO: Include S3ImageField
    # thumbnail = ImageWithThumbsField(upload_to=get_image_path, sizes=(('s', 500, 500), ('r', 450, 600), ('t', 200, 260)), null=True, blank=True)
    notes_todos = models.TextField(_('Notes and Todos'), blank=True)
    auth_data = JSONField(verbose_name=_('author data'), default={}, blank=True,
                          help_text=_("Don't edit this unless you know what this means"))
    public_data = JSONField(verbose_name=_('public data'), default={}, blank=True,
                            help_text=_("Don't edit this unless you know what this means"))
    author = models.ForeignKey(settings.AUTH_USER_MODEL, limit_choices_to=cms_authors)
    created_on = models.DateTimeField(_('created on'), auto_now_add=True)
    modified_on = models.DateTimeField(_('modified on'), auto_now=True)
    objects = ContentManager()

    class Meta:
        ordering = ('-publish',)
        get_latest_by = 'publish'

    @property
    def is_public(self):
        return self.status == 2

    def publish_cms_content(self):
        self.public_data = self.auth_data
        self.status = 2
        self.save()

    def __unicode__(self):
        return self.title

    @property
    @cacheable('cfblog_content_{id}')
    def html(self):
        return get_public_data(self)

    @stale_cache('cfblog_content_{id}')
    def save(self, *args, **kwargs):
        super(Content, self).save(*args, **kwargs)

    def get_previous_post(self):
        return self.get_previous_by_publish(Q(status__gte=2), ~Q(category_id=1))

    def get_next_post(self):
        return self.get_next_by_publish(Q(status__gte=2), ~Q(category_id=1))

    def get_absolute_url(self):
        return self.url



# -*- coding: utf-8 -*-
from setuptools import setup

try:
    long_description = open('README.rst', 'rt').read()
except Exception:
    long_description = ""

setup(
    name="cfblog",
    description="Awesome CMS for Django",
    long_description=long_description,

    version="0.1.8",
    author='Vinay Karanam',
    author_email="vinayinvicible@gmail.com",

    url='https://github.com/Coverfox/cfblog/',
    license='BSD',

    install_requires=[
        "beautifulsoup4",
        "Django>=1.7,<1.9",
        "jsonfield",
        "django-tagging",
        "mistune",
    ],
    packages=['cfblog', 'cfblog.migrations'],

    zip_safe=True,
)



__author__ = 'vinay'
from django.contrib import admin

from .models import Content, Category


class CmsPageCategoryAdmin(admin.ModelAdmin):
    def has_delete_permission(self, request, obj=None):
        if obj is not None and obj.id == 1:
            return False
        return super(CmsPageCategoryAdmin, self).has_delete_permission(request, obj)

admin.site.register(Category, CmsPageCategoryAdmin)


class ContentAdmin(admin.ModelAdmin):
    list_display = ('url', 'title', 'publish', 'status', 'link')
    list_filter = ('publish', 'category', 'status')
    search_fields = ('title', 'url', 'template')
    readonly_fields = ('created_on', 'modified_on')

    def link(self, obj):
        return "<a target='_blank' href='%s'>%s</a>" % (obj.get_absolute_url(), "View on site")
    link.allow_tags = True

admin.site.register(Content, ContentAdmin)



__author__ = 'vinay'
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string


def _get_setting(setting, default_value):
    setting_value = getattr(settings, setting, default_value)

    if setting_value is default_value:
        return setting_value

    if not isinstance(setting_value, type(default_value)) and isinstance(setting_value, (unicode, str)):
        return import_string(setting_value)

    raise ImproperlyConfigured(
        '{} should be of type {} or should be a dotted module path, '
        'instead got {}'.format(setting, type(setting_value), type(default_value))
    )


PAGE_CACHE_TIMEOUT = _get_setting('PAGE_CACHE_TIMEOUT', 60 * 60)
